//
// enums
//

typedef enum
{
	BUP_SUCCESS = 0,
	BUP_NOT_CONNECTED = 1,
	BUP_NOT_FORMATTED = 2,
	BUP_WRITE_PROTECTED = 3,
	BUP_NOT_ENOUGH_SPACE = 4,
	BUP_NOT_FOUND = 5,
	BUP_FOUND = 6,
	BUP_NO_MATCH = 7,
	BUP_BROKEN = 8,
}BUP_ERROR;

typedef enum
{
	OVERWRITE_OFF = 0,
	OVERWRITE_ON = 1,
}BACKUP_OVERWRITE_MODE;

typedef enum
{
	DEVICE_INTERNAL = 0,
	DEVICE_CARTRIDGE = 1,
	DEVICE_SERIAL = 2,
}BACKUP_DEVICE;

typedef enum
{
	DEFAULT = 0,
	SAVE_SEARCH = 1,
	RESUME_SEARCH = 2,
}SEARCH_MODE;

//
// structs
//

struct SERIAL_STRUCT
{
	byte unit_id;
	byte partition_id;
	ushort None;
	undefined4 None;
	undefined4 None;
	undefined2 None;
	undefined2 saved_to_im;
	undefined4 None;
};
typedef SERIAL_STRUCT SERIAL_STRUCT;

struct backup_stat
{
	dword totalsize;
	dword totalblock;
	dword blocksize;
	dword freesize;
	dword freeblock;
	dword datanum;
};
typedef backup_stat backup_stat;

struct INTERNAL_METADATA
{
	pointer[11] function_pointers;
	short * sat_blocks; // Created by retype action
	byte * backup_ram_start;
	dword cart_format_flag?;
	dword totalBlocks;
	dword blockSize;
	dword blockSize_in_bytes;
	dword offset_of_filedata_in_blocks;
	undefined1 bad_block; // Created by retype action
	undefined None;
	undefined None;
	undefined None;
	dword backup_ram_bad_blocks_bitmap;
	dword last_found_block_search; // Created by retype action
	byte internal_connected;
	byte cartridge_connected;
	byte serial_connected;
	undefined None;
	dword totalBlocks_cart;
	dword blockSize_cart;
	dword cartridge_offset_to_data_in_blocks;
	dword cartridge_flag_2;
	dword something_serial?;
	dword serial_int_mask??; // Created by retype action
	undefined None;
	undefined None;
	undefined None;
	undefined None;
	byte serial_partition_number;
	undefined None;
	undefined None;
	undefined None;
};
typedef INTERNAL_METADATA INTERNAL_METADATA;

struct backup_config
{
	word unit_id;
	word partition;
};
typedef backup_config backup_config;

struct backup_date
{
	byte year;
	byte month;
	byte day;
	byte time;
	byte min;
	byte week;
};
typedef backup_date backup_date;

struct backup_file
{
	byte[12] filename;
	byte[11] comment;
	byte language;
	dword date;
	dword datasize;
	dword blocksize;
};
typedef backup_file backup_file;

//
// function prototypes
//

int serial_stat(dword *freesize,dword *freeblock,dword *datanum);
BUP_ERROR BUP_Delete(BACKUP_DEVICE device,uchar *filename);
BUP_ERROR ChecksumData(short block_id,byte *input_data);
undefined4 find_free_blocks??(uint blocks_needed?);
int checkFormatted(void);
void convert_days_to_month_date(uint days,uchar *month,uchar *day,uchar leap_year);
longlong crazyDiv2(int divisor,int dividend);
void setupPowersOf2Struct(byte *powers);
int Serial_SetPartition(SERIAL_STRUCT *param_1,char partition_number);
BUP_ERROR BUP_Dir(BACKUP_DEVICE device,uchar *filename,uint output_size,backup_file *file_metadata);
undefined4 serial_send_data2??(byte *param_1,uint size);
BUP_ERROR BUP_SelPart(BACKUP_DEVICE device,ushort partition_number);
uint serial_send_cmd??(char cmd??,byte *param_2,byte *out_result??);
undefined4 MaskInterrupts_ABus2?(void);
uint somethingSerial3??(int param_1,int param_2,byte *param_3);
uint serial_deleteFile(char *param_1);
undefined4 MaskInterrupts_ABus?(void);
uint software_divide?(int divisor,int dividend);
BUP_ERROR ReadByBlockId(word fileID,byte *outBuf);
uint Serial_Read_error??(char *outBuf);
int serial_read_file2(int param_1,int *param_2,byte *param_3,char param_4);
int something_serial5(int param_1,int *param_2,char *param_3);
void InitBackupRamFormatString(char *backupFormatStr);
void DeleteFileByID(uint block);
BUP_ERROR BUP_Stat(BACKUP_DEVICE device,uint datasize,backup_stat *output_stat);
void Init(INTERNAL_METADATA *im_out,backup_config *config);
undefined4 WaitLoop??(void);
int Serial_ProcessError?(int serial_error);
BUP_ERROR BUP_Write(BACKUP_DEVICE device,backup_file *file,uchar *data,BACKUP_OVERWRITE_MODE overwrite_mode);
BUP_ERROR BUP_Read(BACKUP_DEVICE device,uchar *filename,uchar *output_data);
undefined4 serial_write2(int param_1,int param_2,undefined param_3);
undefined4 serial_read_data???(undefined *buf,uint size??,uint param_3);
int serial_send_data_chksum_??(byte *buf,uint size);
int serial_format(char param_1);
longlong software_modulo?(void);
undefined4 mark_bad_blocks???(void);
void BUP_GetDate(int minutes_since_1980,backup_date *date);
int crazyDiv2(int divisor,int dividend);
void SetOnChipReg?(undefined4 param_1);
void setup_days_by_month(dword *months);
uint serial_write(int param_1,int param_2);
uint serial_readfile(char *filename,byte *out,char param_3);
BUP_ERROR BUP_Verify(BACKUP_DEVICE device,uchar *filename,uchar *data);
int CheckForSerial(backup_config *config);
int CountReservedBlocks(void);
uint SearchForSaveBlock(char *filename,SEARCH_MODE search_mode);
uint BUP_SetDate(backup_date *date);
BUP_ERROR BUP_Format(BACKUP_DEVICE device);

